import plotly.graph_objects as go
from plotly.subplots import make_subplots
import numpy as np
import plotly.express as px

def imshow(
        img,
        zmin=None,
        zmax=None,
        origin=None,
        color_continuous_scale=None,
        color_continuous_midpoint=None,
        range_color=None,
        title=None,
        template=None,
        width=None,
        height=None
           ):
    return px.imshow(
        img,
        zmin=None,
        zmax=None,
        origin=None,
        color_continuous_scale=None,
        color_continuous_midpoint=None,
        range_color=None,
        title=None,
        template=None,
        width=None,
        height=None)
imshow.__doc__ = px.imshow.__doc__


class Subplots:
    def __init__(self, rows=1, cols=1, **kwargs):
        self.rows = rows
        self.cols = cols
        x_title = kwargs.get('x_title', 'x_title')
        y_title = kwargs.get('y_title', 'y_title')
        self.fig = make_subplots(rows=rows, cols=cols, x_title=x_title, y_title=y_title)
        self.count = 0

    def plot(self, x, y, **kwargs):
        '''
        Params:
            --label: look like matplotlib's legend
            --mode: 'lines', 'markers', 'lines+markers', 'text'
            --marker_color
            --line_color
            --marker_size

        '''
        label = kwargs.get('label', self.count + 1)
        mode = kwargs.get('mode', 'lines')  #
        line_color = kwargs.get('line_color', None)
        marker_color = kwargs.get('marker_color', None)
        marker_size = kwargs.get('marker_size', None)
        row, col = np.divmod(self.count, self.cols)
        row, col = int(row + 1), int(col + 1)

        self.fig.add_trace(go.Scatter(x=x, y=y, mode=mode, marker_color=marker_color,
                                      line_color=line_color,
                                      marker_size=marker_size,
                                      name=label),
                           row=row, col=col)

        self.count += 1

    def show(self, *args, **kwargs):
        self.fig.show(*args, **kwargs)


class Multiplots:
    def __init__(self, **kwargs):
        #         self.title = 'title'
        self.title = 'fig.title'
        self.fig = go.Figure()
        self.xlabel = 'fig.xlabel'
        self.ylabel = 'fig.ylabel'
        self.count = 0
        self.annotations = []

    def plot(self, *args, **kwargs):
        '''
        Params:
            :param label: look like matplotlib's legend.
            :param mode: 'lines'(default), 'markers', 'lines+markers', 'text'
            :param line_width: corresponding lines mode.
            :param marker_size: corresponding of markers mode.
            :param text: hovering text
        '''
        if (len(args) >1 and type(args[0]) != type(args[1])) or len(args)==1:
            y = args[0]
            x = np.arange(len(y))
        else:
            y = args[1]
            x = args[0]

        label = kwargs.get('label', self.count + 1)
        mode = kwargs.get('mode', 'lines')  #
        line_width = kwargs.get('line_width', None)
        marker_size = kwargs.get('marker_size', None)
        text = kwargs.get('text', f'f(x)')
        self.fig.add_trace(go.Scatter(x=x, y=y,
                                      mode=mode,
                                      name=label,
                                      text=text,
                                      marker_line_width=line_width,
                                      marker_size=marker_size))
        self.count += 1

    def show(self, *args, **kwargs):
        Multiplots.show.__doc__ = self.fig.show.__doc__


        self.annotations.append(dict(xref='paper', yref='paper', x=0.0, y=1.05,  # title
                                     xanchor='left', yanchor='bottom',
                                     text=self.title,
                                     font=dict(family='Arial',
                                               size=30,
                                               color='rgb(37,37,37)'),
                                     showarrow=False))
        #
        self.annotations.append(dict(xref='paper', yref='paper', x=0.5, y=-0.1,  # X axis
                                     xanchor='center', yanchor='top',
                                     text=self.xlabel,
                                     font=dict(family='Arial',
                                               size=15,
                                               color='rgb(150,150,150)'),
                                     showarrow=False))

        self.fig.update_layout(
            # title=f'{self.title:}',
            overwrite=False,

            yaxis=dict(
                title=self.ylabel,
                showgrid=True,

                zeroline=False,
                showline=True,
                showticklabels=True,),

            xaxis=dict(
                # title=self.xlabel,
                showline=True,
                showgrid=True,
                showticklabels=True,
                linecolor='rgb(204, 204, 204)',
                linewidth=2,
                ticks='outside',
                tickfont=dict(
                    family='Arial',
                    size=12,
                    color='rgb(82, 82, 82)'),
                        ),
            annotations=self.annotations
        )
        self.fig.show(*args, 
                      config={'editable': True,
                            'scrollZoom': False,
                             }, 
                      **kwargs)

    def write_html(self, *args, **kwargs):
        Multiplots.write_html.__doc__ = self.fig.write_html.__doc__
        return self.fig.write_html(*args, **kwargs)

    def write_image(self, *args, **kwargs):
        Multiplots.write_image.__doc__ = self.fig.write_image.__doc__
        return self.fig.write_image(*args, **kwargs)


class Scatter3d():
    def __init__(self, **kwargs):
        self.fig = go.Figure()


    def scatter3d(self,x, y, z, mode=None, color_line=None,color_marker=None,marker_size=4,line_width=2):
        """
        Params:
            :param mode: 'lines', 'markers', 'lines+markers'(default), 'text'
            :param marker_size: corresponding of markers mode.
        """
        if color_marker is None:
            color_marker = None#z
        if color_line is None:
            color_line = None#'darkblue'

        self.fig.add_trace(go.Scatter3d(x=x,y=y,z=z, 
                                        mode=mode,
                                        
                                        marker=dict(
                                        size=marker_size,
                                        color=color_marker,
                                        colorscale='Viridis',
                                        ),
                                        
                                        line=dict(
                                            color=color_line,
                                            width=line_width
                                                  ),
                                    ))
    def show(self,*args, **kwargs):
        
        self.fig.update_layout(
            width=800,
            height=700,
            autosize=False,
            scene=dict(
                camera=dict(
                    up=dict(
                        x=0,
                        y=0,
                        z=1
                    ),
                    eye=dict(
                        x=0,
                        y=1.0707,
                        z=1,
                    )
                ),
                aspectratio = dict( x=1, y=1, z=0.7 ),
                aspectmode = 'manual'
            ),
        )
        self.fig.show(*args, 
                      config={'editable': True,
                            'scrollZoom': True,
                             },**kwargs)

    def write_html(self, *args, **kwargs):
        Multiplots.write_html.__doc__ = self.fig.write_html.__doc__
        return self.fig.write_html(*args, **kwargs)

    def write_image(self, *args, **kwargs):
        Multiplots.write_image.__doc__ = self.fig.write_image.__doc__
        return self.fig.write_image(*args, **kwargs)

class Surface():
    def __init__(self, **kwargs):
        self.fig = go.Figure()


    def surface(self,**kwargs):
        """
        point x= y= z= 
        or z=
        x, y, z should be 2D array.
        """

        self.fig.add_surface(**kwargs)
        
    def show(self,*args, **kwargs):
        
        self.fig.update_layout(
            width=800,
            height=700,
            autosize=False,
            margin=dict(l=65, r=50, b=65, t=90), 

        )
        self.fig.show(*args, 
                      config={'editable': True,
                            'scrollZoom': True,
                             },**kwargs)

    def write_html(self, *args, **kwargs):
        Multiplots.write_html.__doc__ = self.fig.write_html.__doc__
        return self.fig.write_html(*args, **kwargs)

    def write_image(self, *args, **kwargs):
        Multiplots.write_image.__doc__ = self.fig.write_image.__doc__
        return self.fig.write_image(*args, **kwargs)


import itchat
from guang.wechat.Utils import download_file, dynamic_specified_msg, get_userName,d_time
import argparse

def downloads(nickName='caloi', fileType='mp3', d_t=60):
    """ downloads wechat files with any file type
    Args:
        d_t: Program run duration. unit: s
    """
    itchat.auto_login(hotReload=True, enableCmdQR=2)

    while d_time(d_t):
        
        msg = dynamic_specified_msg(get_userName(nickName)[nickName])
        msg, tpath = download_file(msg, fileType=fileType)
        print(tpath)
        
if __name__=="__main__":
    downloads()



import os
import guang.version as version

def get_version(update=False):
    v= version.version
    if update:
        v += 1
        with open(os.path.join('guang','version.py'),'w') as fo:
            fo.write('version= '+ str(v))
    return '.'.join(list(str(v)))




'''
This is a universal function library.
'''

import guang.get_version as get_version

__v = get_version.get_version(update=False)

# '0.0.7.2.7'
__version__ = '0.0.' + __v
print('guang.__version__=:', __version__)
__author__ = 'K.y'
__copyright = 'Copyright 2019 K.y'

__all__ = ["ML","Utils","DL", "Voice", "wechat", "ML", "interesting", "app"]




import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn
import numpy as np

mpl.rcParams['ytick.direction'] = 'in'
mpl.rc('mathtext',fontset = 'cm',rm = 'serif')
mpl.rc(["xtick"], direction="in", top=1)
mpl.rc(["ytick"], direction="in", right=1)
# mpl.rc('font', family='Times New Roman', size=12)
mpl.rc('lines', linewidth=1)
seaborn.set_style('white')


def fillplot(*args, scalex=True, scaley=True, data=None, **kwargs):
    fillplot.__doc__ = 'fill_y_min:阴影下界, fill_y_max: 阴影上界\n' + plt.plot.__doc__
    if type(args[0]) != type(args[1]):
        y = args[0]
        x = np.arange(len(y))
    else:
        y = args[1]
        x = args[0]
    fill_y_min = kwargs.get('fill_y_min', 0)
    fill_y_max = kwargs.get('fill_y_max', y) 

    plt.plot(*args, scalex=True, scaley=True, data=None, **kwargs)
    plt.fill_between(x, y1=fill_y_min, y2=fill_y_max, color='b', alpha=.2)
   



import numpy as np
import Levenshtein # pip install python-Levenshtein

'''
Some common distance function
'''

def euclidean_dist(vec1, vec2):

    assert vec1.shape == vec2.shape 
    return np.sqrt(np.sum((vec1 - vec2) ** 2))


def manhattan_dist(vec1, vec2):

    return np.sum(np.abs(vec1 - vec2))


def chebyshev_dist(vec1, vec2):

    return np.max(np.abs(vec1 - vec2))

def minkowski_dist(vec1, vec2, p=2):
    """
    inputParam: p
    return distance,
    while p=1: dist = manhattan_dist
    while p=2: dist = euclidean_dist
    while p=inf: dist = chebyshev_dist
    """
    s = np.sum(np.power(vec2 - vec1, p))
    return np.power(s,1/p)


def cosine_dist(vec1, vec2):

    # np.linalg.norm(vec, ord=1) 计算p=1范数,默认p=2
    return (vec1.T @ vec2)/(np.linalg.norm(vec1) * np.linalg.norm(vec2))


def distance(a, b):
    '''字符串距离'''
    return Levenshtein.ratio(a, b)


def hamming_dist(x, y):
    return np.sum(x != y) / len(x)


def jaccard_simil_coef():
    pass


import string
import re 

useless = string.printable + "，。；’：‘’“”！？《》￥…（）—·【】、"


def has_chinese(str0):
    """判断是否存在汉字
    """
    return bool(re.search(u'[\u4e00-\u9fa5]', str0))
# def is_chinese(uchar):
#     """判断一个unicode是否是汉字"""
#     if uchar >= u'\u4e00' and uchar<=u'\u9fa5':
#         return True
#     else:
#         return False
def has_english(str0):
    return bool(re.search('[a-z]', str0.lower()))

def has_number(str0):
    return bool(re.search('[0-9]', str0))

def replace_bracket(a):
    l0 = a.find('（')
    l1 = a.find('）')
    a=a.replace(a[l0:l1+1],'')
    return a

def sub2empty(useless, str0):
    return re.sub(f"[{useless}]+", "", str0)

if __name__ == "__main__":
    print(useless)


from IPython.display import Audio, display
import soundfile as sf
from librosa.display import waveplot, specshow
import numpy as np
import matplotlib.pyplot as plt

def listen(*voice, dtype='float64', samplerate=None):
    """
    voice can be numpy array + fs
    or file path
    use soundfile packege ,it provide audio data from a sound file as NumPy array."""
    if type(voice[0]) == str:
        x, fs = sf.read(voice[0], dtype=dtype, samplerate=samplerate)
    elif type(voice[0])== np.ndarray:
        if len(voice) < 2:
            raise ValueError('Invalid inputs, pls input both np.array and fs ')
        x, fs = voice[0], voice[1]
    else:
        raise ValueError('invalid input, voice can be np.array+fs or file path')

    return display(Audio(x, rate=fs))


def voiceshow(*voice, samplerate=None, dtype='float64', figsize=(10, 4)):
    """
    voice can be numpy array + fs
    or file path
    give the voice wave plot.
    """
    if type(voice[0]) == str:
        x, fs = sf.read(voice[0], dtype=dtype, samplerate=samplerate)
    elif type(voice[0])== np.ndarray:
        if len(voice) < 2:
            raise ValueError('Invalid inputs, pls input both np.array and fs ')
        x, fs = voice[0], voice[1]
    else:
        raise ValueError('invalid inputs, voice can be np.array+fs or file path')
    plt.figure(figsize=figsize)
    waveplot(x, fs)
    plt.show()

def duration(y=None,
            sr=22050,
            S=None,
            n_fft=2048,
            hop_length=512,
            center=True,
            filename=None,):
    """Compute the duration (in seconds) of an audio time series, feature matrix, or filename.
    return librosa.get_duration
    """
    return librosa.get_duration(y=y,
                                sr=sr,
                                S=S,
                                n_fft=n_fft,
                                hop_length=hop_length,
                                center=center,
                                filename=filename,)


from guang.Utils.toolsFunc import probar



import numpy as np 

def transform(X, dim):
	"""
	A principal component analysis model.
	
	Parameters
	----------
	X : :py:class:`ndarray <numpy.ndarray>` of shape `(Samples,  characteristics)`
		The input samples.
	dim : int
		Take the previous dim principal components.

	return:
	-------
	shape = (Samples, dim)
	"""
	U, S, V = np.linalg.svd(X-X.mean(0),full_matrices=True)
	idx = S.argsort()[::-1][:dim]
	return X.dot(V[idx].T)



import pickle
import itchat
from itchat.content import *
import time
from glob import glob
import os
from guang.Utils.toolsFunc import path
from datetime import datetime

def get_all_info():
    """get all my friends' information """
    nickNames = []

    for friend in itchat.get_friends():
        nickNames.append(friend.NickName)
        # print(nickNames)
    return nickNames


def get_userName(*name_list):
    UserNames = {}
    for name in name_list:
        friends = itchat.search_friends(name=name)

        if len(friends) == 1:
            UserNames[name] = friends[0].UserName
        else:
            print(f'name: “{name}” is NOT unique')
            raise Exception('')
    return UserNames


def save_userName():
    name_list = ['爸爸', '妈妈',
                 '被冻结的光', '光']
    userNames = get_userName(*name_list)
    with open('userNames.pkl', 'wb') as fo:
        pickle.dump(userNames, fo)


def load_userName():
    with open('userNames.pkl', 'rb') as fi:
        userNames = pickle.load(fi)


def get_list(lis=[''], count=[0]):
    '''get msg list and count'''

    @itchat.msg_register([TEXT, MAP, CARD, NOTE, SHARING, PICTURE, RECORDING, ATTACHMENT, VIDEO])
    def reply(msg):
        lis[0] = msg
        count[0] = count[0] + 1
        return

    return lis[0], count[0]


count_n = 0


def get_content(l, count):
    global count_n
    if count == count_n:
        count_n += 1
        return l
    else:
        return False


def get_msg():
    #     global = count_n
    l, count = get_list()
    message = get_content(l, count)
    if message:
        return message
    else:
        return None


def dynamic_msg():
    '''Dynamic acquisition of anyone's message
    Once acquired, the program releases'''
    while 1:
        msg = get_msg()
        if msg == None:
            time.sleep(.6)
            itchat.configured_reply()
            continue
        else:
            return msg


def dynamic_specified_msg(userName=None):
    '''
    Dynamic acquisition of specified user's message
    Once acquired, the program releases
    '''
    while 1:
        msg = get_msg()
        try:
            msg_userName = msg.user.userName
        except:
            msg_userName = None

        if msg_userName == None or msg_userName != userName:
            time.sleep(.6)
            itchat.configured_reply()
            continue
        else:
            return msg


flag_get_txt = True
MY_USER_NAME = None
def get_txt(msg):
    global flag_get_txt, MY_USER_NAME
    if flag_get_txt:
        MY_USER_NAME = get_userName("被冻结的光")["被冻结的光"]
        # print(MY_USER_NAME)
        flag_get_txt = False
    tpath = os.path.join('wechat_download', msg.User.NickName, 'text')
    if not os.path.exists(tpath):
        os.makedirs(tpath)
    nowTime = datetime.now()
    nowTime = nowTime.strftime("%Y-%m-%d %H:%M:%S")
    if msg['FromUserName'] != MY_USER_NAME:
        text = f"{msg.User.NickName: <10}:{msg.text: <100} {nowTime: >10} \n"
    else:
        text = f"{'被冻结的光': <10}:{msg.text: <100} {nowTime: >10}\n"
    with open(path(tpath + '/dialogue.txt'), 'a+', encoding='utf-8') as fw:
        fw.write(text)
    return msg


def download_file(msg, fileType='mp3'):
    '''
    Param
    -----
        fileType : 'attachment' ,'mp3', 'mp4', 'png'
        and ATTACHMENT,  of which include various file types.
    Returns
    -------
        msg, file_path
    '''
    #     msg = dynamic_specified_msg(username)
    file_name = msg.fileName.split('.')
    sufname = file_name[-1]
    prename = '.'.join(file_name[:-1])

    if fileType == 'attachment':  # 'attachment':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'attachment')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))
    elif sufname == 'png' and fileType == 'png':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'picture')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))
    elif sufname == 'mp3' and fileType == 'mp3':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'voice')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))
    elif sufname == 'mp4' and fileType == 'mp4':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'video')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))

    #     os.remove(msg.fileName) # remove 当前文件夹下的下载文件
    return msg, tpath


def d_time(d_t, t0=[]):
    """
    The unit of dt is seconds.
    Returns 1 when current time in `d_t` duration
    Returns 0 when current time out duration
    """
    if not t0:
        t0.append(time.time())
    t1 = time.time()
    if t1 - t0[0] < d_t:
        return True
    else:
        return False


if __name__ == '__main__':
    itchat.auto_login(hotReload=True)

    while d_time(60):
        msg = dynamic_specified_msg(get_userName('光')['光'])
        msg = download_file(msg, fileType='mp3')





import numpy as np 
import cv2
from glob import glob
from tensorflow.keras.datasets import mnist
def load_mnist(SIZE=128,num_train=2000, num_test=500):
	'''
	SIZE: points image size
	'''

	(x_train, _), (x_test, _) = mnist.load_data()
	x_train = x_train.astype('float32') / 255.
	x_test = x_test.astype('float32') / 255.
	x_train = np.reshape(x_train, (len(x_train), 28, 28, 1))  
	x_test = np.reshape(x_test, (len(x_test), 28, 28, 1)) 

	X_train, X_test = x_train[0][...,-1], x_test[0][...,-1]

	X_train, X_test = cv2.resize(X_train, (SIZE,SIZE)), cv2.resize(X_test, (SIZE,SIZE))
	X_train, X_test = X_train.reshape(1,SIZE,SIZE,1), X_test.reshape(1,SIZE,SIZE,1)
	for i in x_train[:num_train]:
		x_i = cv2.resize(i[...,-1], (SIZE, SIZE))
		X_train = np.concatenate((X_train, x_i.reshape(1,SIZE,SIZE,1)), axis=0)
		
	for j in x_test[-num_train:]:
		x_j = cv2.resize(j[...,-1], (SIZE, SIZE))
		X_test = np.concatenate((X_test, x_j.reshape(1,SIZE,SIZE,1)), axis=0)
	return X_train, X_test


def load_path(img_path = '/home/nikki_intern/GAN/DATA/img_align_celeba/*', SIZE = 256, 
              num_train=2000, num_test=1000):

    img_name = glob(img_path)
    train_name = img_name[:2000]
    test_name = img_name[-500:]

    X_train = cv2.resize(cv2.imread(train_name[0])[:,:,::-1], (SIZE,SIZE)) 
    X_test = cv2.resize(cv2.imread(test_name[0])[:,:,::-1], (SIZE,SIZE))
    H,W,C = X_train.shape
    X_train, X_test = X_train.reshape(1,SIZE,SIZE,C), X_test.reshape(1,SIZE,SIZE,C)
    for i in train_name[1:]:
        x_i = cv2.resize(cv2.imread(i)[:,:,::-1], (SIZE,SIZE)) 
        X_train = np.concatenate((X_train, x_i.reshape(1,SIZE,SIZE,C)), axis=0)
        
    for j in test_name[1:]:
        x_j = cv2.resize(cv2.imread(j)[:,:,::-1], (SIZE,SIZE)) 
        X_test = np.concatenate((X_test, x_j.reshape(1,SIZE,SIZE,C)), axis=0)
        
    return X_train.astype('float32') / 255., X_test.astype('float32') / 255.


import matplotlib.pyplot as plt
import cv2
import numpy as np

def cvt2rgb(img,channel = 'bgr'):
    '''it can convert image channel BGR,BGRA,HLS,HSV to RGB'''
    if channel=='bgr' or channel=='BGR' or channel=='BGRA':
        print('bgr',img.shape)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    elif channel=='HLS' or channel=='hls':
        img = cv2.cvtColor(img, cv2.COLOR_HLS2RGB)
    elif channel=='HSV' or channel=='hsv':
        img = cv2.cvtColor(img, cv2.COLOR_HSV2RGB)
    else:
        return 'bad channel'
    return img
    
def myplot(img,channel='bgr'):
    ''' can show image channel BGR,BGRA,HLS,HSV.
    if RGB, set channel = 0'''
    if channel:
        img = cvt2rgb(img, channel)
    plt.figure(figsize=(7,7))
    plt.imshow(img)
    plt.show()



# 查看图像HSV直方图
def get_img_hist(imgName):
    '''
    :param: imgName: the path of image
    :return: 
    '''
    img = cv2.imread(imgName)

    img = cv2.resize(img,(100,100),interpolation=cv2.INTER_CUBIC)
    H,S,V = cv2.split(cv2.cvtColor(img,cv2.COLOR_BGR2HSV))

    hist_h = np.bincount(H.ravel(), minlength=180) 
    hist_s = np.bincount(S.ravel(), minlength=256) 
    hist_v = np.bincount(V.ravel(), minlength=256) 
    
    hist_h = hist_h/ np.sum(hist_h) # normalization
    hist_s = hist_s/ np.sum(hist_s) # normalization
    hist_v = hist_v/ np.sum(hist_v) # normalization

    plt.figure(figsize=(9,9))
    plt.subplot(3,1,1); plt.plot(hist_h); plt.title('H')
    plt.subplot(3,1,2); plt.plot(hist_s); plt.title('S')
    plt.subplot(3,1,3); plt.plot(hist_v); plt.title('V')
    plt.show()


from multiprocessing import Pool, TimeoutError
import os
import librosa
from glob import glob
import soundfile as sf
import numpy as np
import warnings

def is_lt_duration(file, least_time):
    try:
        if librosa.get_duration(filename = file) < least_time:
            return 1
        else:
            return 0
    except:
        print(f'file {i} open failed, and has been ignored')
#         return None

def reduce_from_duration(filelist, least_time=1):
    '''
    return reduced list
    '''
    with Pool(processes=None) as pool:
        multi_res=[pool.apply_async(is_lt_duration, (i,least_time)) for i in filelist]
        reduce_list = [res.get() for res in multi_res]
        
    return [filelist[idx] for idx,i in enumerate(reduce_list) if i==0]

def find_no_silence(filename):
    """read the sound file, return you the voice start end voice end.
    
    Return:
        start(/s), end(/s), arg_start, arg_end
    """
    warnings.filterwarnings('ignore')
    voice, sr = sf.read(filename)
#     diff = np.diff(voice, n=2)
#     print(diff)
#     plt.plot(diff)
#     print(np.argwhere(diff>1e-4))
    argvoice = np.argwhere(np.log(voice, )>-6)
    a= argvoice/sr
    win = int(16 * sr/20000)
    for i in range(len(a)):
        if abs(np.mean(a[i:i+win])-a[i]) < 1e-3:
            start = a[i]
            arg_start = argvoice[i]
            break

    for i in range(-1, -len(a), -1):
        if abs(np.mean(a[i-win:i])-a[i]) < 1e-3:
            end = a[i-win]
            arg_end = argvoice[i-win]
            break
    
    return start[0], end[0], arg_start[0], arg_end[0]

import time
if __name__ == "__main__":
    pass
#     t1=time.time()
#     flist = glob(r"data/*")
#     l = reduce_from_duration(flist, least_time=3)
#     print(l)
#     print(time.time()- t1)



# tensorflow2.0
import tensorflow as tf
from tensorflow.keras import Input,Model
from tensorflow.keras.layers import Conv2D,MaxPooling2D, UpSampling2D


class CAE:
	def __init__(self,w, h, input_channel):
	 self.w, self.h, self.input_channel = w, h, input_channel

	def compile(self):
		input_img = Input(shape=(self.w,self.h,self.input_channel))
		x = Conv2D(16, (3, 3),strides=(2,2), activation='relu', padding='same')(input_img)
		x = Conv2D(8, (3, 3), strides=(2,2), activation='relu', padding='same')(x)
		encoded = Conv2D(8, (3, 3), strides=(2,2), activation='relu', padding='same')(x)

		x = Conv2D(8, (3, 3), activation='relu', padding='same')(encoded)
		t_b, t_h,t_w, t_c = x.shape
		x = tf.image.resize(x, ( t_w*2, t_h*2))
		x = Conv2D(8, (3, 3), activation='relu', padding='same')(x)
		t_b, t_h,t_w, t_c = x.shape
		x = tf.image.resize(x, ( t_w*2, t_h*2))
		x = Conv2D(16, (3, 3), activation='relu', padding='same')(x)
		t_b, t_h,t_w, t_c = x.shape
		x = tf.image.resize(x, ( t_w*2, t_h*2))
		decoded = Conv2D(1, (3, 3), activation='sigmoid', padding='same')(x)

		self.autoencoder = Model(input_img, decoded)
		self.autoencoder.compile(optimizer='adam', loss='binary_crossentropy') 
	
	def fit(self,x_train, x_test, epochs=5, batch_size=128):

		self.autoencoder.fit(x_train, x_train,
						epochs=epochs,
						batch_size=batch_size,
						shuffle=True,
						validation_data=(x_test, x_test)
					)


def TEST(SIZE=64):
	'''
	SIZE: Specify image size
	'''
	import sys
	sys.path.append('../..')
	from Utils.load_data import load_mnist
	import matplotlib.pyplot as plt
	x_train, x_test = load_mnist(SIZE=SIZE)
	tb,th,tw,tc = x_test.shape
	w, h, input_channel = th,tw,1
	cae = CAE(w, h, input_channel)
	cae.compile()
	cae.fit(x_train, x_test,epochs=1)

	decoded_imgs = cae.autoencoder.predict(x_test[:20])

	n = 5                                        
	plt.figure(figsize=(10, 4))
	for i in range(1,n+1):
		# display original
		ax = plt.subplot(2, n, i)
		plt.imshow(x_test[i].reshape(w,h))
		plt.gray()
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)

		# display reconstruction
		ax = plt.subplot(2, n,i+n)
		plt.imshow(decoded_imgs[i].reshape(w,h))
		plt.gray()
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)
	plt.show()
if __name__=='__main__':
	TEST()


 # -*- coding:utf-8 -*-
from __future__ import print_function
from setuptools import setup, find_packages
import guang
from glob import glob


with open(glob('requirements.*')[0], encoding='utf-8') as f:
    all_reqs = f.read().split('\n')
install_requires = [x.strip() for x in all_reqs if 'git+' not in x]

with open("README.md", "r", encoding='utf-8') as fr:
    long_description = fr.read()


setup(
       name = "guang" ,
       version=guang.__version__,
       description = " Some tools function" ,

       long_description=long_description,
       long_description_content_type="text/markdown",
       author = "K.y" ,
       author_email="beidongjiedeguang@gmail.com",
       url = "https://github.com/beidongjiedeguang/guang" ,
       license = "GPL-v3" ,
       packages = find_packages(exclude=['tests*', 'kaldi*']),
       install_requires=install_requires,
       classifiers=[
	     'Operating System :: OS Independent',
	     'Intended Audience :: Developers',
	     'Intended Audience :: Science/Research',
	     'Topic :: Scientific/Engineering :: Artificial Intelligence',
       "Programming Language :: Python :: 3",
       "Programming Language :: Python :: 3.5",
       "Programming Language :: Python :: 3.6",
       "Programming Language :: Python :: 3.7",
       ],
       keywords=[
          'Deep Learning',
          'Machine Learning',
          'Neural Networks',
          'Natural Language Processing',
          'Computer Vision'
      ]
)


from datetime import datetime, timedelta, timezone

class beijing:
    # utc_time = datetime.utcnow()
    utc_time = datetime.now(tz=timezone.utc)
    bj_time = utc_time.astimezone(timezone(timedelta(hours=8)))
    now = bj_time.now()


def finish_time(percent, hours, minutes, seconds):
    bj_time = beijing.bj_time
    time_spent = timedelta(hours=hours, minutes=minutes, seconds=seconds)

    all_time = time_spent/(percent/100)
    time_remain = all_time - time_spent
    end_time = bj_time + time_remain
    print(f"训练结束时间 {end_time:%Y}年 {end_time:%m}月 {end_time:%d}日 {end_time:%H}点 {end_time:%M}分")
    
    


import os
from pydub import AudioSegment
from multiprocessing import Pool

def cvt2wav(orig_path, new_path, sr=16000):
    basename = os.path.basename(orig_path)
    sufname = basename.split('.')[-1]

    x = AudioSegment.from_file(orig_path, format=sufname)
    x = x.set_channels(1)
    x = x.set_frame_rate(sr)
    x = x.set_sample_width(2)
    x.export(new_path, format='wav')

# def multi_cvt2wav(PATH_orig, PATH_new, sr=16000, n_cpu=None):
#     with Pool(processes=n_cpu) as pool:
#         for i,j in zip(PATH_orig, PATH_new):
#             pool.apply_async(cvt2wav, (i, j, sr))
#         pool.join()

    
def multi_cvt2wav(PATH_orig, PATH_new, sr=16000, n_cpu=16):
    pool =Pool(processes=n_cpu)
    for i,j in zip(PATH_orig, PATH_new):
        pool.apply_async(cvt2wav, (i, j, sr))
    pool.close()
    pool.join()

if __name__ == "__main__":
    pass
